;; wax-seal.scm -*-scheme-*-
;; Wax Seal - create wax-seal-looking object from original picture
;;
;; This is a version compatible with GIMP v1.1.20
;;
;; Copyright (C) 2000 by Jaroslav Benkovsky <Edheldil@atlas.cz>
;; Released under General Public License (GPL)
;;
;; $Id: wax-seal.scm,v 1.3 2000/06/26 01:02:31 benkovsk Exp $
;;
;; Requires:
;;    plug-in-plasma
;;    plug-in-emboss
;;    plug-in-gauss-rle
;;
;; source image is rgb or grayscale, with black-on-white picture

(define (script-fu-wax-seal img drawable wax-color 
			    light-azimuth light-elevation light-depth 
			    border-thickness border-threshold 
			    inner-hollow? symbol-separate? symbol-thickness 
			    bump? bump-granularity bump-smoothness bump-seed
			    highlight-size highlight-start highlight-smoothness)

  (let* ((drawable (car (gimp-image-get-active-layer img)))
	     (width (car (gimp-drawable-width drawable)))
	     (height (car (gimp-drawable-height drawable)))
	     (aliassel TRUE)
	     (old-fg (car (gimp-context-get-foreground)))
	     (old-bg (car (gimp-context-get-background)))
	     (type (car (gimp-drawable-type drawable)))
	     (alpha-type (cond ((= type RGB-IMAGE) RGBA-IMAGE)
			               ((= type RGBA-IMAGE) RGBA-IMAGE)
			               ((= type GRAY-IMAGE) GRAYA-IMAGE)
			               ((= type GRAYA-IMAGE) GRAYA-IMAGE)))
	     (color-layer (car (gimp-layer-new img width height alpha-type "color layer" 100  NORMAL-MODE)))
	     (mask-layer (car (gimp-layer-new img width height alpha-type "mask layer" 100  NORMAL-MODE)))
         (symbol-layer (car (gimp-layer-copy drawable TRUE)))
         (maska 0)
         (highlight-layer 0) )

    ; setup
    (gimp-image-undo-group-start img)
    (gimp-selection-none img)
    (gimp-context-set-foreground '(0 0 0))
    (gimp-context-set-background '(255 255 255))

    ; get rid of transparent parts
;    (if (= (gimp-drawable-has-alpha drawable) TRUE)
;	(begin
;	  (gimp-selection-layer-alpha drawable)
;	  (gimp-selection-invert img)
;	  (gimp-edit-bucket-fill drawable BG-BUCKET-FILL NORMAL-MODE 100 0 FALSE 0 0)))

    (gimp-drawable-fill color-layer TRANSPARENT-FILL)
    (gimp-drawable-fill mask-layer WHITE-FILL)


    ; separating inner symbol
    (if (= symbol-separate? TRUE)
	(begin
	    (gimp-image-add-layer img symbol-layer -1)
	    (gimp-fuzzy-select symbol-layer 1 1 (* 3 border-threshold) CHANNEL-OP-ADD TRUE FALSE 0 FALSE)
	    (gimp-selection-grow img 1)
	    (gimp-edit-bucket-fill symbol-layer FG-BUCKET-FILL NORMAL-MODE 100 0 FALSE 0 0)
	    (gimp-selection-none img)
	    (gimp-fuzzy-select symbol-layer 1 1 (* 3 border-threshold) CHANNEL-OP-ADD TRUE FALSE 0 FALSE)
	    (gimp-selection-grow img 1)

	    (gimp-by-color-select symbol-layer '(255 255 255) (* 0.3 border-threshold) CHANNEL-OP-ADD FALSE FALSE 0 FALSE)

	    (gimp-edit-clear symbol-layer)
	    (gimp-selection-invert img)
	    (gimp-edit-bucket-fill drawable BG-BUCKET-FILL NORMAL-MODE 100 0 FALSE 0 0)
	    (gimp-selection-none img)
	    (if (> symbol-thickness 0)
		(plug-in-gauss-rle 1 img symbol-layer symbol-thickness TRUE TRUE))
	    ))


    ; prepare seal border for emboss
    (if (> border-thickness 0)
	(begin
	    (set! drawable (car (gimp-image-get-active-drawable img)))
	    (plug-in-gauss-rle 1 img drawable border-thickness TRUE TRUE)))

    ; bumping seal top surface
    (if (= bump? TRUE)
	(begin
	    (gimp-image-add-layer img mask-layer -1)
	    (set! maska (car (gimp-layer-create-mask mask-layer ADD-WHITE-MASK)))
	    (gimp-layer-add-mask mask-layer maska)
	    (plug-in-plasma 1 img maska bump-seed bump-granularity)
	    (if (> bump-smoothness 0)
		(plug-in-gauss-rle 1 img maska bump-smoothness TRUE TRUE))
	    (gimp-layer-set-opacity mask-layer 50)
	    (set! drawable (car (gimp-image-flatten img)))

	    ;(gimp-brightness-contrast drawable 30 60)
        (define (set-pt a index x y)
          (begin
            (aset a (* index 2) x)
            (aset a (+ (* index 2) 1) y))
        )
        (define (spline-selection-wax-seal)
          (let* ((a (cons-array 8 'byte)))
                    (set-pt a 0 0 0)
                    (set-pt a 1 65 65)
                    (set-pt a 2 180 220)
                    (set-pt a 3 255 255)
                  a
        ))
        (gimp-curves-spline drawable 0 8 (spline-selection-wax-seal))
	    ))

    ; reincluding inner symbol
    (if (= symbol-separate? TRUE) 
	    (set! drawable (car (gimp-image-flatten img))))

    ; embossing the seal - monochrome
    (if (= inner-hollow? FALSE)
	(begin 
	  (gimp-fuzzy-select drawable 1 1 border-threshold CHANNEL-OP-ADD TRUE FALSE 0 FALSE))
	(begin
	  (gimp-by-color-select drawable '(255 255 255) border-threshold CHANNEL-OP-ADD TRUE FALSE 0 FALSE)))

    (gimp-selection-invert img)
    (plug-in-emboss 1 img drawable light-azimuth light-elevation light-depth TRUE)

    ; create layer with wax color - colorize the seal
    (gimp-layer-add-alpha drawable)
    (gimp-layer-set-mode drawable VALUE-MODE)
    (gimp-drawable-set-name drawable "seal layer")
    (gimp-image-add-layer img color-layer 1)
    (gimp-context-set-foreground wax-color)
    (gimp-edit-bucket-fill color-layer FG-BUCKET-FILL NORMAL-MODE 100 0 FALSE 0 0)
    (set! drawable (car (gimp-image-merge-visible-layers img EXPAND-AS-NECESSARY)))

    ; try to create highlights
    (if (> highlight-size 0)
	(begin
	  (if (> (+ highlight-start highlight-size) 256)
	      (set! highlight-start (- 256 highlight-size)))

	  (set! highlight-layer (car (gimp-layer-copy drawable 0)))
	  (gimp-image-add-layer img highlight-layer -1)
	  (gimp-threshold highlight-layer highlight-start 
			  (+ highlight-start highlight-size))
	  (if (> highlight-smoothness 0)
	      (plug-in-gauss-rle 1 img highlight-layer highlight-smoothness TRUE TRUE))
	  (gimp-layer-set-mode highlight-layer LIGHTEN-ONLY-MODE)
	  (gimp-image-merge-visible-layers img EXPAND-AS-NECESSARY)))


    ; cleanup
    (gimp-selection-none img)
    (gimp-context-set-foreground old-fg)
    (gimp-context-set-background old-bg)
    (gimp-image-undo-group-end img)
    (gimp-displays-flush)
))

(define (script-fu-wax-text-logo text size font wax-color bg-color create-shadow?)
    (let* (
	   ; script parameters' defs
	   (light-azimuth 315)
	   (light-elevation 45)
	   (light-depth 20)
	   (bump-granularity 4)
	   (bump-smoothness 10)
	   (bump-seed 0)
	   (border (/ size 4))
	   (wax-thickness (/ size 10))
	   (shadow-depth (/ size 15))
	   (highlight-size 0)
	   (highlight-start 235)
	   (highlight-smoothness 5)

	   (old-fg (car (gimp-context-get-foreground)))
	   (old-bg (car (gimp-context-get-background)))
      	   (dummy (gimp-context-set-foreground '(0 0 0)))

	   (img (car (gimp-image-new 256 256 RGB)))
	   (text-layer (car (gimp-text-fontname img -1 0 0 text border TRUE size PIXELS font)))
	   (width (car (gimp-drawable-width text-layer)))
	   (height (car (gimp-drawable-height text-layer)))
	   (text-layer 0) 
	   (layers 0)
	   (layer-cnt 0)
	   (text-layer 0)
	   (bg-layer 0) )

    ; setup
    (gimp-image-undo-disable img)
    (gimp-image-resize img width height 0 0)

    ;(if (= create-highlight? FALSE)
	;(set! highlight-size 0))

    ; "waxify" the text
    (set! text-layer (car (gimp-image-flatten img)))
    (script-fu-wax-seal img text-layer wax-color 
			light-azimuth light-elevation light-depth
			wax-thickness 20 TRUE FALSE 0 TRUE 
			bump-granularity bump-smoothness bump-seed
			highlight-size highlight-start highlight-smoothness)

    (set! layers (gimp-image-get-layers img))
    (set! layer-cnt (car layers))
    (set! text-layer (aref (cadr layers) 0))

    ; add background layer and color
    (set! bg-layer (car (gimp-layer-new img width height RGB-IMAGE "Background" 100 NORMAL-MODE)))
    (gimp-image-add-layer img bg-layer layer-cnt)
    (gimp-context-set-background bg-color)
    (gimp-edit-fill bg-layer BACKGROUND-FILL)


    ; add shadow layer
    (if (= create-shadow? TRUE)
	(begin
	  (set! bg-layer (car (gimp-layer-new img width height RGBA-IMAGE "Shadow Layer" 75 NORMAL-MODE)))
	  (gimp-image-add-layer img bg-layer layer-cnt)
	  (gimp-edit-clear bg-layer)
	  (gimp-selection-layer-alpha text-layer)
	  (gimp-edit-bucket-fill bg-layer FG-BUCKET-FILL NORMAL-MODE 100 100 FALSE 0 0)
	  (gimp-selection-none img)
	  (plug-in-mblur 1 img bg-layer 0 shadow-depth (- light-azimuth 90) 0 0)
	  (plug-in-gauss-rle 1 img bg-layer shadow-depth TRUE TRUE)
	  ))

    ; cleanup
    (gimp-context-set-foreground old-fg)
    (gimp-context-set-background old-bg)
    (gimp-image-undo-enable img)
    (gimp-display-new img)
))



(script-fu-register "script-fu-wax-text-logo"
		    "Woskowe..."
		    "Woskowe logo"
		    "Jaroslav Benkovsky <Edheldil@atlas.cz>"
		    "Jaroslav Benkovsky"
		    "June 2000"
		    ""
		    SF-STRING     "Tekst"               "Wax Text"
		    SF-ADJUSTMENT "Rozmiar czcionki (px)" '(100 2 1000 1 10 0 1)
		    SF-FONT       "Czcionka"               "Becker"
		    SF-COLOR      "Kolor"             '(170 13 13)
		    SF-COLOR      "Kolor tła"             '(255 255 255)
		    SF-TOGGLE     "Tworzenie cienia"           TRUE)

(script-fu-menu-register "script-fu-wax-text-logo"
  "<Image>/File/Create/Logos")

