; Wire Art Text rel 0.03
; Created by Graechan
;
;    Created from this video tutorial at http://www.youtube.com/watch?v=N-3YuZlZn7Q&feature=related
;    that inspired me to write this script. 
; Comments directed to http://gimpchat.com or http://gimpscripts.com
;
; License: GPLv3
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;    GNU General Public License for more details.
;
;    To view a copy of the GNU General Public License
;    visit: http://www.gnu.org/licenses/gpl.html
;
;
; ------------
;| Change Log |
; ------------ 
; Rel 0.01 - Initial Release
; Rel 0.02 - Added multi-line text with justification and spacing
;            Changed ellipse to circular bkg to fit multi-line text
;            Added choice to add bkg to image only
; Rel 0.03 - Included the ellipse shape
;
(define (script-fu-wire-art-text text
                                 just
                                 space								 
                                 font-in 
                                 font-size
							     grow
							     shape
							     wire-size
							     metal
							     metal-text
							     tint-color
                                 bkg-color
								 bkg-crop)
                                 
  (let* (
         (img-width 250)
		 (img-height 250)
		 (width img-width)
         (height img-height)
         (offx 0)
         (offy 0)
         (image (car (gimp-image-new img-width img-height RGB)))
         (border (/ font-size 3))
		 (font (if (> (string-length font-in) 0) font-in (car (gimp-context-get-font))))
         (text-layer (car (gimp-text-fontname image -1 0 0 text border TRUE font-size PIXELS font)))
         (text-width (car (gimp-drawable-width text-layer)))
         (text-height (car (gimp-drawable-height text-layer)))
		 (bkg-layer 0)
		 (selection-channel 0)
		 (img-channel 0)
         (tint-layer 0)        
         )
		 
    (gimp-context-push)
	(gimp-context-set-foreground '(0 0 0))
	(gimp-context-set-background '(255 255 255))    
    
;;;;position the text lines
	(if (= just 0) (gimp-text-layer-set-justification text-layer 2))
	(if (= just 1) (gimp-text-layer-set-justification text-layer 0))
	(if (= just 2) (gimp-text-layer-set-justification text-layer 1))
	(gimp-text-layer-set-letter-spacing text-layer (/ font-size 12))
	(gimp-text-layer-set-line-spacing text-layer space) 
	(gimp-text-layer-set-letter-spacing text-layer (/ font-size 12))
	(set! text-width (car (gimp-drawable-width text-layer)))
    (set! text-height (car (gimp-drawable-height text-layer)))
	
;;;;set the new Image size
	(if (> text-width img-width) (set! width text-width))           
    (if (> text-height img-height) (set! height text-height))
	
;;;;square up the Image if needed
	(if (= shape 2) 
	(begin
    (if (< height width) (set! height width))
	(if (< width height) (set! width height))))
	
;;;;resize the image	
	(gimp-image-resize image width height 0 0)   
;;;;centre the text layer	
    (set! offx (/ (- width text-width) 2))
    (set! offy (/ (- height text-height) 2))    
    (gimp-layer-set-offsets text-layer offx offy)	

;;;;Expand the font if needed
    (if (> grow 0)
           (begin
    (gimp-selection-layer-alpha text-layer)
    (gimp-edit-clear text-layer)
    (gimp-selection-grow image grow)
    (gimp-context-set-foreground '(0 0 0))
    (gimp-edit-fill text-layer FOREGROUND-FILL)
    (gimp-selection-none image)
           )
    )	
	(set! bkg-layer (car (gimp-layer-new image width height RGBA-IMAGE "Background" 100 NORMAL-MODE)))
    (gimp-image-add-layer image bkg-layer 1)
	
	(if (> shape 0) (gimp-ellipse-select image 0 0 (- width 5) (- height 5) 2 TRUE FALSE 0))
	(if (= shape 0) 
	(begin
	(gimp-selection-all image)
	(gimp-selection-shrink image 10)
	(script-fu-selection-rounded-rectangle image bkg-layer 50 FALSE)
	)
	)
;;;;create channel
	(gimp-selection-save image)
	(set! selection-channel (car (gimp-image-get-active-drawable image)))	
	(gimp-channel-set-opacity selection-channel 100)	
	(gimp-drawable-set-name selection-channel "Shape Selection")
	(gimp-image-set-active-layer image bkg-layer)
	
	(gimp-edit-fill bkg-layer FOREGROUND-FILL)
	(gimp-selection-shrink image wire-size)
	(gimp-edit-clear bkg-layer)
	
	(plug-in-grid 1 image bkg-layer 2 16 8 '(0 0 0) 255 2 16 8 '(0 0 0) 255 0 2 6 '(0 0 0) 255)
	(gimp-selection-layer-alpha text-layer)
;;;;create channel
	(gimp-selection-save image)
	(set! img-channel (car (gimp-image-get-active-drawable image)))	
	(gimp-channel-set-opacity img-channel 100)	
	(gimp-drawable-set-name img-channel "img-channel")
	(gimp-image-set-active-layer image text-layer)	
    (gimp-selection-shrink image wire-size)
	(set! text-layer (car (gimp-image-merge-down image text-layer EXPAND-AS-NECESSARY)))
	(gimp-drawable-set-name text-layer "Wire Text")
	(gimp-edit-clear text-layer)
    (gimp-selection-none image)
	
	(script-fu-chrome-logo-alpha image text-layer 10 bkg-color)
	
	(if (= metal 1) (gimp-context-set-foreground '(255 215 0)));gold 	
    (if (= metal 2) (gimp-context-set-foreground '(192 192 192)));silver	
    (if (= metal 3) (gimp-context-set-foreground '(250 180 150)));copper
    (if (= metal 4) (gimp-context-set-foreground '(166 125 61)));bronze
    (if (= metal 5) (gimp-context-set-foreground tint-color));colored tint
    (if (> metal 0)
    (begin 
	(set! tint-layer (car (gimp-layer-new image width height RGBA-IMAGE "Tint" 100 MULTIPLY-MODE)))
    (gimp-image-add-layer image tint-layer -1)
    (if (= metal-text TRUE) (gimp-selection-load img-channel))	
	(gimp-edit-fill tint-layer FOREGROUND-FILL)
	(set! text-layer (car (gimp-image-merge-down image tint-layer CLIP-TO-IMAGE)))
	(if (< metal 5) (gimp-brightness-contrast text-layer 50 50))
	(gimp-selection-none image)
	)
	)
	(if (= metal 0)
    (begin 
	(set! tint-layer (car (gimp-layer-new image width height RGBA-IMAGE "Tint" 100 MULTIPLY-MODE)))
    (gimp-image-add-layer image tint-layer -1) 
	(set! text-layer (car (gimp-image-merge-down image tint-layer CLIP-TO-IMAGE)))
	)
	)
	(set! text-layer (car (gimp-image-merge-down image text-layer EXPAND-AS-NECESSARY)))
	(gimp-drawable-set-name text-layer "Wire Text")
	
	(plug-in-autocrop 1 image text-layer)
	
	(set! bkg-layer (car (gimp-layer-new image width height RGBA-IMAGE "Tint" 100 MULTIPLY-MODE)))
    (gimp-image-add-layer image bkg-layer 1) 
	(set! bkg-layer (car (gimp-image-merge-down image bkg-layer CLIP-TO-IMAGE)))
	
	(if (= bkg-crop TRUE)
	(begin
	(gimp-selection-load selection-channel)
	(gimp-selection-invert image)
	(gimp-edit-clear bkg-layer)
    (gimp-selection-none image)))
	(gimp-selection-load img-channel)
	(gimp-context-set-foreground bkg-color)
    (gimp-edit-fill bkg-layer FOREGROUND-FILL)
    (gimp-selection-none image)
		
    (gimp-context-pop)      
    (gimp-display-new image)
	
    )
  )
  
(script-fu-register "script-fu-wire-art-text"
  "Wire Art Text"
  "Creates text made fom wire"
  "Graechan"
  "Graechan"
  "June 2012"
  ""
  SF-TEXT       "Text"    "Wire\nArt\nText" 
  SF-OPTION "Justification" '("Centre" "Left" "Right")
  SF-ADJUSTMENT "Line Spacing (pixels)" '(0 -8192 8192 1 10 0 0)
  SF-FONT       "Font"               "Sans Bold"
  SF-ADJUSTMENT "Font size (pixels)" '(180 100 300 1 1 0 0)
  SF-ADJUSTMENT "Expand the Font if needed" '(3 0 6 1 1 0 1)
  SF-OPTION "Image Shape" '("Rectangular" "Ellipse" "Circular")
  SF-ADJUSTMENT "Wire Size (pixels)" '(5 2 10 1 1 0 0)
  SF-OPTION "Metal Plating Type" '("Chrome" "Gold" "Silver" "Copper" "Bronze" "Colored Tint")
  SF-TOGGLE     "Plate the Text only"   FALSE
  SF-COLOR      "Tint Color"         '(0 0 255)
  SF-COLOR      "Background color"         "lightgrey"
  SF-TOGGLE     "Bacground to Image only"   FALSE  
  )
(script-fu-menu-register "script-fu-wire-art-text"
  "<Image>/File/Create/Logos"
  )